@@ -32,13 +32,16 @@
 from __future__ import annotations
 
 import re
+import unicodedata
 import warnings
 import weakref
 
-from collections import namedtuple
+from collections import deque, namedtuple
 from decimal import Decimal
 from typing import (
+    TYPE_CHECKING,
     Any,
+    Deque,
     Dict,
     Generator,
     Iterator,
@@ -47,12 +50,10 @@ from typing import (
     Optional,
     Sequence,
     Tuple,
-    Type,
     Union,
 )
-from weakref import CallableProxyType
 
-from .abstracts import NAMED_TUPLE_CACHE, MySQLConnectionAbstract, MySQLCursorAbstract
+from .abstracts import NAMED_TUPLE_CACHE, MySQLCursorAbstract
 from .constants import ServerFlag
 from .errors import (
     Error,
@@ -64,16 +65,21 @@ from .errors import (
 from .types import (
     DescriptionType,
     EofPacketType,
+    MySQLConvertibleType,
     ParamsDictType,
     ParamsSequenceOrDictType,
     ParamsSequenceType,
     ResultType,
+    RowItemType,
     RowType,
     StrOrBytes,
-    ToPythonOutputTypes,
     WarningType,
 )
 
+if TYPE_CHECKING:
+    from .connection import MySQLConnection
+
+
 SQL_COMMENT = r"\/\*.*?\*\/"
 RE_SQL_COMMENT = re.compile(
     rf"""({SQL_COMMENT})|(["'`][^"'`]*?({SQL_COMMENT})[^"'`]*?["'`])""",
@@ -111,6 +117,71 @@ ERR_NO_RESULT_TO_FETCH = "No result set to fetch from"
 MAX_RESULTS = 4294967295
 
 
+def is_eol_comment(stmt: bytes) -> bool:
+    """Checks if statement is an end-of-line comment.
+
+    Double-dash comment style requires the second dash to be
+    followed by at least one whitespace (Z) or control character (C) such
+    as a space, tab, newline, and so on.
+
+    Hash comment simply requires start from `#` and nothing else.
+
+    Args:
+        stmt: MySQL statement.
+
+    Returns:
+        Whether or not the statement is an end-of-line comment.
+
+    References:
+        [1]: https://dev.mysql.com/doc/refman/8.0/en/comments.html
+    """
+    is_double_dash_comment = (
+        len(stmt) >= 3
+        and stmt.startswith(b"--")
+        and unicodedata.category(chr(stmt[2]))[0] in {"Z", "C"}
+    )
+    is_hash_comment = len(stmt) >= 2 and stmt.startswith(b"#")
+
+    return is_double_dash_comment or is_hash_comment
+
+
+def parse_multi_statement_query(multi_stmt: bytes) -> Deque[bytes]:
+    """Parses a multi-statement query/operation.
+
+    Parsing consists of removing empty (which includes just whitespaces and/or control
+    characters) statements and EOL (end-of-line) comments.
+
+    However, there's a caveat, by rule, the last EOL comment found in the stream isn't
+    removed if and only if it's the last statement.
+
+    Why? EOL comments do not generate results, however, when the last statement is an
+    EOL comment the server returns an empty result. So, in other to match statements
+    and results correctly we need to keep the last EOL comment statement.
+
+    Args:
+        multi_stmt: Query representing multi-statement operations separated by semicolons.
+
+    Returns:
+        A list of statements that aren't empty and don't contain leading
+        ASCII whitespaces. Also, they aren't EOL comments except
+        perhaps for the last one.
+    """
+    executed_list: Deque[bytes] = deque(RE_SQL_SPLIT_STMTS.split(multi_stmt))
+    stmt, num_stms = b"", len(executed_list)
+    while num_stms > 0:
+        num_stms -= 1
+        stmt_next = executed_list.popleft().lstrip()
+        if stmt_next:
+            stmt = stmt_next
+            if not is_eol_comment(stmt):
+                executed_list.append(stmt)
+
+    if is_eol_comment(stmt):
+        executed_list.append(stmt)
+
+    return executed_list
+
+
 class _ParamSubstitutor:
     """
     Substitutes parameters into SQL statement.
@@ -185,7 +256,9 @@ class CursorBase(MySQLCursorAbstract):
         self.arraysize: int = 1
         super().__init__()
 
-    def callproc(self, procname: str, args: Sequence[Any] = ()) -> Any:
+    def callproc(
+        self, procname: str, args: Sequence = ()
+    ) -> Optional[Union[Dict[str, RowItemType], RowType]]:
         """Calls a stored procedue with the given arguments
 
         The arguments will be set during this session, meaning
@@ -209,15 +282,17 @@ class CursorBase(MySQLCursorAbstract):
         Raises exceptions when something is wrong.
         """
 
-    def close(self) -> Any:
+    def close(self) -> None:
         """Close the cursor."""
 
     def execute(
         self,
-        operation: Any,
-        params: Union[Sequence[Any], Dict[str, Any]] = (),
+        operation: str,
+        params: Union[
+            Sequence[MySQLConvertibleType], Dict[str, MySQLConvertibleType]
+        ] = (),
         multi: bool = False,
-    ) -> Any:
+    ) -> Optional[Generator[MySQLCursorAbstract, None, None]]:
         """Executes the given operation
 
         Executes the given operation substituting any markers with
@@ -237,8 +312,12 @@ class CursorBase(MySQLCursorAbstract):
         """
 
     def executemany(
-        self, operation: Any, seq_params: Sequence[Union[Sequence[Any], Dict[str, Any]]]
-    ) -> Any:
+        self,
+        operation: str,
+        seq_params: Sequence[
+            Union[Sequence[MySQLConvertibleType], Dict[str, MySQLConvertibleType]]
+        ],
+    ) -> Optional[Generator[MySQLCursorAbstract, None, None]]:
         """Execute the given operation multiple times
 
         The executemany() method will execute the operation iterating
@@ -261,14 +340,14 @@ class CursorBase(MySQLCursorAbstract):
         data using the execute() method.
         """
 
-    def fetchone(self) -> Optional[Sequence[Any]]:
+    def fetchone(self) -> Optional[Union[RowType, Dict[str, RowItemType]]]:
         """Return next row of a query result set.
 
         Returns:
             tuple or None: A row from query result set.
         """
 
-    def fetchmany(self, size: int = 1) -> List[Sequence[Any]]:
+    def fetchmany(self, size: int = 1) -> List[Union[RowType, Dict[str, RowItemType]]]:
         """Return the next set of rows of a query result set.
 
         When no more rows are available, it returns an empty list.
@@ -279,24 +358,43 @@ class CursorBase(MySQLCursorAbstract):
             list: The next set of rows of a query result set.
         """
 
-    def fetchall(self) -> List[Sequence[Any]]:
+    def fetchall(self) -> List[Union[RowType, Dict[str, RowItemType]]]:
         """Return all rows of a query result set.
 
         Returns:
             list: A list of tuples with all rows of a query result set.
         """
 
-    def nextset(self) -> Any:
+    def stored_results(self) -> Iterator[MySQLCursorAbstract]:
+        """Returns an iterator for stored results.
+
+        This method returns an iterator over results which are stored when
+        callproc() is called. The iterator will provide `MySQLCursorBuffered`
+        instances.
+
+        Examples:
+            ```
+            >>> cursor.callproc('myproc')
+            ()
+            >>> for result in cursor.stored_results():
+            ...     print result.fetchall()
+            ...
+            [(1,)]
+            [(2,)]
+            ```
+        """
+
+    def nextset(self) -> NoReturn:
         """Not Implemented."""
 
-    def setinputsizes(self, sizes: Any) -> Any:
+    def setinputsizes(self, sizes: Any) -> NoReturn:
         """Not Implemented."""
 
-    def setoutputsize(self, size: Any, column: Any = None) -> Any:
+    def setoutputsize(self, size: Any, column: Any = None) -> NoReturn:
         """Not Implemented."""
 
-    def reset(self, free: bool = True) -> Any:
-        """Reset the cursor to default"""
+    def reset(self, free: bool = True) -> None:
+        """Resets the cursor to default"""
 
     @property
     def description(self) -> Optional[List[DescriptionType]]:
@@ -349,11 +447,9 @@ class MySQLCursor(CursorBase):
     Implements the Python Database API Specification v2.0 (PEP-249)
     """
 
-    def __init__(
-        self, connection: Optional[Type[MySQLConnectionAbstract]] = None
-    ) -> None:
+    def __init__(self, connection: Optional[MySQLConnection] = None) -> None:
         CursorBase.__init__(self)
-        self._connection: CallableProxyType[Type[MySQLConnectionAbstract]] = None
+        self._connection: Optional[MySQLConnection] = None
         self._nextrow: Tuple[Optional[RowType], Optional[EofPacketType]] = (
             None,
             None,
@@ -370,7 +466,7 @@ class MySQLCursor(CursorBase):
         """
         return iter(self.fetchone, None)
 
-    def _set_connection(self, connection: Type[MySQLConnectionAbstract]) -> None:
+    def _set_connection(self, connection: MySQLConnection) -> None:
         """Set the connection"""
         try:
             self._connection = weakref.proxy(connection)
@@ -386,8 +482,8 @@ class MySQLCursor(CursorBase):
         self._warnings: Optional[List[WarningType]] = None
         self._warning_count: int = 0
         self._description: Optional[List[DescriptionType]] = None
-        self._executed: Optional[StrOrBytes] = None
-        self._executed_list: List[StrOrBytes] = []
+        self._executed: Optional[bytes] = None
+        self._executed_list: List[bytes] = []
         self.reset()
 
     def _have_unread_result(self) -> bool:
@@ -525,27 +621,62 @@ class MySQLCursor(CursorBase):
         """Generator returns MySQLCursor objects for multiple statements
 
         This method is only used when multiple statements are executed
-        by the execute() method. It uses zip() to make an iterator from the
-        given query_iter (result of MySQLConnection.cmd_query_iter()) and
-        the list of statements that were executed.
+        by the `cursor.execute(multi_stmt_query, multi=True)` method.
+
+        It matches the given `query_iter` (result of `MySQLConnection.cmd_query_iter()`)
+        and the list of statements that were executed.
+
+        How does this method work? To properly map each statement (stmt) to a result,
+        the following facts must be considered:
+
+        1. Read operations such as `SELECT` produce a non-empty result
+            (calling `next(query_iter)` gets a result that includes at least one column).
+        2. Write operatios such as `INSERT` produce an empty result
+            (calling `next(query_iter)` gets a result with no columns - aka empty).
+        3. End-of-line (EOL) comments do not produce a result, unless is the last stmt
+            in which case produces an empty result.
+        4. Calling procedures such as `CALL my_proc` produce a sequence `(1)*0` which
+            means it may produce zero or more non-empty results followed by just one
+            empty result. In other words, a callproc stmt always terminates with an
+            empty result. E.g., `my_proc` includes an update + select + select + update,
+            then the result sequence will be `110` - note how the write ops results get
+            annulated, just the read ops results are produced. Other examples:
+                * insert + insert -> 0
+                * select + select + insert + select -> 1110
+                * select -> 10
+            Observe how 0 indicates the end of the result sequence. This property is
+            vital to know what result corresponds to what callproc stmt.
+
+        In this regard, the implementation is composed of:
+        1. Parsing: the multi-statement is broken down into single statements, and then
+            for each of these, leading white spaces are removed (including
+            jumping line, vertical line, tab, etc.). Also, EOL comments are removed from
+            the stream, except when the comment is the last statement of the
+            multi-statement string.
+        2. Mapping: the facts described above as used as "game rules" to properly match
+        statements and results. In case, if we run out of statements before running out
+        of results we use a sentinel named "stmt_overflow!" to indicate that the mapping
+        went wrong.
+
+        Acronyms
+            1: a non-empty result
+            2: an empty result
         """
-        executed_list = RE_SQL_SPLIT_STMTS.split(self._executed)
+        executed_list = parse_multi_statement_query(multi_stmt=self._executed)
+        self._executed = None
+        stmt = executed_list.popleft() if executed_list else b"stmt_overflow!"
+        for result in query_iter:
+            self._reset_result()
+            self._handle_result(result)
 
-        i = 0
-        while True:
-            try:
-                result = next(query_iter)
-                self._reset_result()
-                self._handle_result(result)
-                try:
-                    self._executed = executed_list[i].strip()
-                    i += 1
-                except IndexError:
-                    self._executed = executed_list[0]
+            if is_eol_comment(stmt):
+                continue
 
-                yield self
-            except StopIteration:
-                return
+            self._executed = stmt.rstrip()
+            yield self
+
+            if not stmt.upper().startswith(b"CALL") or "columns" not in result:
+                stmt = executed_list.popleft() if executed_list else b"stmt_overflow!"
 
     def execute(
         self,
@@ -748,8 +879,8 @@ class MySQLCursor(CursorBase):
     def callproc(
         self,
         procname: str,
-        args: Sequence[Any] = (),
-    ) -> Optional[Union[Dict[str, ToPythonOutputTypes], RowType]]:
+        args: Sequence = (),
+    ) -> Optional[Union[Dict[str, RowItemType], RowType]]:
         """Calls a stored procedure with the given arguments
 
         The arguments will be set during this session, meaning
@@ -895,7 +1026,7 @@ class MySQLCursor(CursorBase):
             raise InterfaceError(f"Failed getting warnings; {err}") from None
 
         if res:
-            return res
+            return res  # type: ignore[return-value]
 
         return None
 
@@ -1034,7 +1165,7 @@ class MySQLCursor(CursorBase):
         if self._executed is None:
             return None
         try:
-            return self._executed.strip().decode("utf-8")  # type: ignore[union-attr]
+            return self._executed.strip().decode("utf-8")
         except (AttributeError, UnicodeDecodeError):
             return self._executed.strip()  # type: ignore[return-value]
 
@@ -1055,7 +1186,7 @@ class MySQLCursor(CursorBase):
         fmt = "{class_name}: {stmt}"
         if self._executed:
             try:
-                executed = self._executed.decode("utf-8")  # type: ignore[union-attr]
+                executed = self._executed.decode("utf-8")
             except AttributeError:
                 executed = self._executed
             if len(executed) > 40:
@@ -1068,9 +1199,7 @@ class MySQLCursor(CursorBase):
 class MySQLCursorBuffered(MySQLCursor):
     """Cursor which fetches rows within execute()"""
 
-    def __init__(
-        self, connection: Optional[Type[MySQLConnectionAbstract]] = None
-    ) -> None:
+    def __init__(self, connection: Optional[MySQLConnection] = None) -> None:
         super().__init__(connection)
         self._rows: Optional[List[RowType]] = None
         self._next_row: int = 0
@@ -1225,7 +1354,7 @@ class MySQLCursorBufferedRaw(MySQLCursorBuffered):
 class MySQLCursorPrepared(MySQLCursor):
     """Cursor using MySQL Prepared Statements"""
 
-    def __init__(self, connection: Optional[Type[MySQLConnectionAbstract]] = None):
+    def __init__(self, connection: Optional[MySQLConnection] = None):
         super().__init__(connection)
         self._rows: Optional[List[RowType]] = None
         self._next_row: int = 0
@@ -1482,14 +1611,14 @@ class MySQLCursorDict(MySQLCursor):
         self,
         rowdata: RowType,
         desc: Optional[List[DescriptionType]] = None,  # pylint: disable=unused-argument
-    ) -> Optional[Dict[str, ToPythonOutputTypes]]:
+    ) -> Optional[Dict[str, RowItemType]]:
         """Convert a MySQL text result row to Python types
 
         Returns a dictionary.
         """
         return dict(zip(self.column_names, rowdata)) if rowdata else None
 
-    def fetchone(self) -> Optional[Dict[str, ToPythonOutputTypes]]:
+    def fetchone(self) -> Optional[Dict[str, RowItemType]]:
         """Return next row of a query result set.
 
         Returns:
@@ -1497,7 +1626,7 @@ class MySQLCursorDict(MySQLCursor):
         """
         return self._row_to_python(super().fetchone(), self.description)
 
-    def fetchall(self) -> List[Optional[Dict[str, ToPythonOutputTypes]]]:
+    def fetchall(self) -> List[Optional[Dict[str, RowItemType]]]:
         """Return all rows of a query result set.
 
         Returns:
@@ -1574,7 +1703,7 @@ class MySQLCursorBufferedDict(MySQLCursorDict, MySQLCursorBuffered):
     Buffered Cursor fetching rows as dictionaries.
     """
 
-    def fetchone(self) -> Optional[Dict[str, ToPythonOutputTypes]]:
+    def fetchone(self) -> Optional[Dict[str, RowItemType]]:
         """Return next row of a query result set.
 
         Returns:
@@ -1586,7 +1715,7 @@ class MySQLCursorBufferedDict(MySQLCursorDict, MySQLCursorBuffered):
             return self._row_to_python(row, self.description)
         return None
 
-    def fetchall(self) -> List[Optional[Dict[str, ToPythonOutputTypes]]]:
+    def fetchall(self) -> List[Optional[Dict[str, RowItemType]]]:
         """Return all rows of a query result set.
 
         Returns:
@@ -1651,9 +1780,7 @@ class MySQLCursorPreparedDict(MySQLCursorDict, MySQLCursorPrepared):  # type: ig
     4. MySQLCursor (base class)
     """
 
-    def fetchmany(
-        self, size: Optional[int] = None
-    ) -> List[Dict[str, ToPythonOutputTypes]]:
+    def fetchmany(self, size: Optional[int] = None) -> List[Dict[str, RowItemType]]:
         """Return the next set of rows of a query result set.
 
         When no more rows are available, it returns an empty list.
