@@ -26,7 +26,7 @@
 # along with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 
-# mypy: disable-error-code="assignment,arg-type,override,union-attr"
+# mypy: disable-error-code="assignment,override,union-attr"
 
 """Cursor classes using the C Extension."""
 from __future__ import annotations
@@ -37,6 +37,7 @@ import weakref
 
 from collections import namedtuple
 from typing import (
+    TYPE_CHECKING,
     Any,
     Dict,
     Generator,
@@ -46,13 +47,11 @@ from typing import (
     Optional,
     Sequence,
     Tuple,
-    Type,
     Union,
 )
-from weakref import CallableProxyType
 
 # pylint: disable=import-error,no-name-in-module
-from _mysql_connector import MySQLInterfaceError, MySQLPrepStmt
+from _mysql_connector import MySQLInterfaceError
 
 from .types import (
     CextEofPacketType,
@@ -60,16 +59,21 @@ from .types import (
     DescriptionType,
     ParamsSequenceOrDictType,
     ParamsSequenceType,
+    RowItemType,
     RowType,
     StrOrBytes,
-    ToPythonOutputTypes,
     WarningType,
 )
 
 # pylint: enable=import-error,no-name-in-module
 # isort: split
 
-from .abstracts import NAMED_TUPLE_CACHE, MySQLConnectionAbstract, MySQLCursorAbstract
+from .abstracts import (
+    NAMED_TUPLE_CACHE,
+    CMySQLPrepStmt,
+    MySQLConnectionAbstract,
+    MySQLCursorAbstract,
+)
 from .cursor import (
     RE_PY_PARAM,
     RE_SQL_COMMENT,
@@ -79,7 +83,8 @@ from .cursor import (
     RE_SQL_ON_DUPLICATE,
     RE_SQL_PYTHON_CAPTURE_PARAM_NAME,
     RE_SQL_PYTHON_REPLACE_PARAM,
-    RE_SQL_SPLIT_STMTS,
+    is_eol_comment,
+    parse_multi_statement_query,
 )
 from .errorcode import CR_NO_RESULT_SET
 from .errors import (
@@ -90,6 +95,9 @@ from .errors import (
     get_mysql_exception,
 )
 
+if TYPE_CHECKING:
+    from .connection_cext import CMySQLConnection
+
 ERR_NO_RESULT_TO_FETCH = "No result set to fetch from"
 
 
@@ -127,7 +135,7 @@ class CMySQLCursor(MySQLCursorAbstract):
     _buffered: bool = False
     _raw_as_string: bool = False
 
-    def __init__(self, connection: Type[MySQLConnectionAbstract]) -> None:
+    def __init__(self, connection: CMySQLConnection) -> None:
         """Initialize"""
         MySQLCursorAbstract.__init__(self)
 
@@ -140,9 +148,7 @@ class CMySQLCursor(MySQLCursorAbstract):
 
         if not isinstance(connection, MySQLConnectionAbstract):
             raise InterfaceError(errno=2048)
-        self._cnx: CallableProxyType[Type[MySQLConnectionAbstract]] = weakref.proxy(
-            connection
-        )
+        self._cnx: CMySQLConnection = weakref.proxy(connection)
 
     def reset(self, free: bool = True) -> None:
         """Reset the cursor
@@ -198,7 +204,7 @@ class CMySQLCursor(MySQLCursorAbstract):
             raise InterfaceError(f"Failed getting warnings; {err}") from None
 
         if warns:
-            return warns
+            return warns  # type: ignore[return-value]
 
         return None
 
@@ -249,22 +255,67 @@ class CMySQLCursor(MySQLCursorAbstract):
             self._cnx.free_result()
 
     def _execute_iter(self) -> Generator[CMySQLCursor, None, None]:
-        """Generator returns MySQLCursor objects for multiple statements
-
-        Deprecated: use nextset() method directly.
+        """Generator returns MySQLCursor objects for multiple statements.
 
         This method is only used when multiple statements are executed
-        by the execute() method. It uses zip() to make an iterator from the
-        given query_iter (result of MySQLConnection.cmd_query_iter()) and
-        the list of statements that were executed.
-        """
-        executed_list = RE_SQL_SPLIT_STMTS.split(self._executed)
-        i = 0
-        self._executed = executed_list[i]
-        yield self
+        by the `cursor.execute(multi_stmt_query, multi=True)` method.
 
+        How does this method work? To properly map each statement (stmt) to a result,
+        the following facts must be considered:
+
+        1. Read operations such as `SELECT` produce a non-empty result
+            (calling `next(query_iter)` gets a result that includes at least one column).
+        2. Write operatios such as `INSERT` produce an empty result
+            (calling `next(query_iter)` gets a result with no columns - aka empty).
+        3. End-of-line (EOL) comments do not produce a result, unless is the last stmt
+            in which case produces an empty result.
+        4. Calling procedures such as `CALL my_proc` produce a sequence `(1)*0` which
+            means it may produce zero or more non-empty results followed by just one
+            empty result. In other words, a callproc stmt always terminates with an
+            empty result. E.g., `my_proc` includes an update + select + select + update,
+            then the result sequence will be `110` - note how the write ops results get
+            annulated, just the read ops results are produced. Other examples:
+                * insert + insert -> 0
+                * select + select + insert + select -> 1110
+                * select -> 10
+            Observe how 0 indicates the end of the result sequence. This property is
+            vital to know what result corresponds to what callproc stmt.
+
+        In this regard, the implementation is composed of:
+        1. Parsing: the multi-statement is broken down into single statements, and then
+            for each of these, leading white spaces are removed (including
+            jumping line, vertical line, tab, etc.). Also, EOL comments are removed from
+            the stream, except when the comment is the last statement of the
+            multi-statement string.
+        2. Mapping: the facts described above as used as "game rules" to properly match
+        statements and results. In case, if we run out of statements before running out
+        of results we use a sentinel named "stmt_overflow!" to indicate that the mapping
+        went wrong.
+
+        Acronyms
+            1: a non-empty result
+            2: an empty result
+        """
+        executed_list = parse_multi_statement_query(multi_stmt=self._executed)
+        self._executed = None
+        stmt = b""
+        result: bool = False
         while True:
             try:
+                if not stmt.upper().startswith(b"CALL") or not result:
+                    stmt = (
+                        executed_list.popleft() if executed_list else b"stmt_overflow!"
+                    )
+
+                # at this point the result has been fetched already
+                result = self._cnx.result_set_available
+
+                if is_eol_comment(stmt):
+                    continue
+
+                self._executed = stmt.rstrip()
+                yield self
+
                 if not self.nextset():
                     raise StopIteration
             except InterfaceError as err:
@@ -273,13 +324,6 @@ class CMySQLCursor(MySQLCursorAbstract):
                     raise
             except StopIteration:
                 return
-            i += 1
-            try:
-                self._executed = executed_list[i].strip()
-            except IndexError:
-                self._executed = executed_list[0]
-            yield self
-        return
 
     def execute(
         self,
@@ -302,7 +346,7 @@ class CMySQLCursor(MySQLCursorAbstract):
             raise ProgrammingError("Cursor is not connected", 2055) from err
         self._cnx.handle_unread_result()
 
-        stmt = ""
+        stmt = b""
         self.reset()
 
         try:
@@ -320,7 +364,7 @@ class CMySQLCursor(MySQLCursorAbstract):
                     stmt = stmt.replace(f"%({key})s".encode(), value)
             elif isinstance(prepared, (list, tuple)):
                 psub = _ParamSubstitutor(prepared)
-                stmt = RE_PY_PARAM.sub(psub, stmt)  # type: ignore[call-overload]
+                stmt = RE_PY_PARAM.sub(psub, stmt)
                 if psub.remaining != 0:
                     raise ProgrammingError(
                         "Not all parameters were used in the SQL statement"
@@ -385,7 +429,9 @@ class CMySQLCursor(MySQLCursorAbstract):
                 prepared = self._cnx.prepare_for_mysql(params)
                 if isinstance(prepared, dict):
                     for key, value in prepared.items():
-                        tmp = tmp.replace(f"%({key})s".encode(), value)
+                        tmp = tmp.replace(
+                            f"%({key})s".encode(), value  # type: ignore[arg-type]
+                        )
                 elif isinstance(prepared, (list, tuple)):
                     psub = _ParamSubstitutor(prepared)
                     tmp = RE_PY_PARAM.sub(psub, tmp)  # type: ignore[call-overload]
@@ -396,7 +442,7 @@ class CMySQLCursor(MySQLCursorAbstract):
                 values.append(tmp)
 
             if fmt in stmt:
-                stmt = stmt.replace(fmt, b",".join(values), 1)
+                stmt = stmt.replace(fmt, b",".join(values), 1)  # type: ignore[arg-type]
                 self._executed = stmt
                 return stmt
             return None
@@ -500,8 +546,8 @@ class CMySQLCursor(MySQLCursorAbstract):
     def callproc(
         self,
         procname: str,
-        args: Sequence[Any] = (),
-    ) -> Optional[Union[Dict[str, ToPythonOutputTypes], RowType]]:
+        args: Sequence = (),
+    ) -> Optional[Union[Dict[str, RowItemType], RowType]]:
         """Calls a stored procedure with the given arguments"""
         if not procname or not isinstance(procname, str):
             raise ValueError("procname must be a string")
@@ -556,7 +602,7 @@ class CMySQLCursor(MySQLCursorAbstract):
                 else:
                     cursor_class = CMySQLCursorBuffered
                 # pylint: disable=protected-access
-                cur = cursor_class(self._cnx.get_self())
+                cur = cursor_class(self._cnx.get_self())  # type: ignore[arg-type]
                 cur._executed = f"(a result of {call})"
                 cur._handle_result(result)
                 # pylint: enable=protected-access
@@ -610,7 +656,7 @@ class CMySQLCursor(MySQLCursorAbstract):
         if not self._cnx.unread_result:
             return []
 
-        rows: Tuple[List[RowType], Optional[CextEofPacketType]] = self._cnx.get_rows()
+        rows = self._cnx.get_rows()
         if self._nextrow and self._nextrow[0]:
             rows[0].insert(0, self._nextrow[0])
 
@@ -701,7 +747,7 @@ class CMySQLCursor(MySQLCursorAbstract):
         Returns a iterator.
         """
         for result in self._stored_results:
-            yield result
+            yield result  # type: ignore[misc]
         self._stored_results = []
 
     def __next__(self) -> RowType:
@@ -776,7 +822,7 @@ class CMySQLCursorBuffered(CMySQLCursor):
 
     """Cursor using C Extension buffering results"""
 
-    def __init__(self, connection: Type[MySQLConnectionAbstract]):
+    def __init__(self, connection: CMySQLConnection):
         """Initialize"""
         super().__init__(connection)
 
@@ -880,7 +926,7 @@ class CMySQLCursorDict(CMySQLCursor):
 
     _raw: bool = False
 
-    def fetchone(self) -> Optional[Dict[str, ToPythonOutputTypes]]:
+    def fetchone(self) -> Optional[Dict[str, RowItemType]]:
         """Return next row of a query result set.
 
         Returns:
@@ -889,7 +935,7 @@ class CMySQLCursorDict(CMySQLCursor):
         row = super().fetchone()
         return dict(zip(self.column_names, row)) if row else None
 
-    def fetchmany(self, size: int = 1) -> List[Dict[str, ToPythonOutputTypes]]:
+    def fetchmany(self, size: int = 1) -> List[Dict[str, RowItemType]]:
         """Return the next set of rows of a query result set.
 
         When no more rows are available, it returns an empty list.
@@ -903,7 +949,7 @@ class CMySQLCursorDict(CMySQLCursor):
         res = super().fetchmany(size=size)
         return [dict(zip(self.column_names, row)) for row in res]
 
-    def fetchall(self) -> List[Dict[str, ToPythonOutputTypes]]:
+    def fetchall(self) -> List[Dict[str, RowItemType]]:
         """Return all rows of a query result set.
 
         Returns:
@@ -919,13 +965,13 @@ class CMySQLCursorBufferedDict(CMySQLCursorBuffered):
 
     _raw = False
 
-    def _fetch_row(self) -> Optional[Dict[str, ToPythonOutputTypes]]:
+    def _fetch_row(self) -> Optional[Dict[str, RowItemType]]:
         row = super()._fetch_row()
         if row:
             return dict(zip(self.column_names, row))
         return None
 
-    def fetchall(self) -> List[Dict[str, ToPythonOutputTypes]]:
+    def fetchall(self) -> List[Dict[str, RowItemType]]:
         """Return all rows of a query result set.
 
         Returns:
@@ -1014,13 +1060,13 @@ class CMySQLCursorBufferedNamedTuple(CMySQLCursorBuffered):
 class CMySQLCursorPrepared(CMySQLCursor):
     """Cursor using MySQL Prepared Statements"""
 
-    def __init__(self, connection: Type[MySQLConnectionAbstract]):
+    def __init__(self, connection: CMySQLConnection):
         super().__init__(connection)
         self._rows: Optional[List[RowType]] = None
         self._rowcount: int = 0
         self._next_row: int = 0
         self._binary: bool = True
-        self._stmt: Optional[MySQLPrepStmt] = None
+        self._stmt: Optional[CMySQLPrepStmt] = None
 
     def _handle_eof(self) -> None:
         """Handle EOF packet"""
