@@ -26,13 +26,18 @@
 # along with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 
+# mypy: disable-error-code="override,attr-defined"
+
 """Database Operations."""
 
+from datetime import datetime, time, timezone
+from typing import Optional
+
 from django.conf import settings
 from django.db.backends.mysql.operations import (
     DatabaseOperations as MySQLDatabaseOperations,
 )
-from django.utils import timezone
+from django.utils import timezone as django_timezone
 
 try:
     from _mysql_connector import datetime_to_mysql, time_to_mysql
@@ -47,7 +52,7 @@ class DatabaseOperations(MySQLDatabaseOperations):
 
     compiler_module = "mysql.connector.django.compiler"
 
-    def regex_lookup(self, lookup_type):
+    def regex_lookup(self, lookup_type: str) -> str:
         """Return the string to use in a query when performing regular
         expression lookup."""
         if self.connection.mysql_version < (8, 0, 0):
@@ -58,17 +63,18 @@ class DatabaseOperations(MySQLDatabaseOperations):
         match_option = "c" if lookup_type == "regex" else "i"
         return f"REGEXP_LIKE(%s, %s, '{match_option}')"
 
-    def adapt_datetimefield_value(self, value):
+    def adapt_datetimefield_value(self, value: Optional[datetime]) -> Optional[bytes]:
         """Transform a datetime value to an object compatible with what is
         expected by the backend driver for datetime columns."""
         return self.value_to_db_datetime(value)
 
-    def value_to_db_datetime(self, value):
+    def value_to_db_datetime(self, value: Optional[datetime]) -> Optional[bytes]:
         """Convert value to MySQL DATETIME."""
+        ans: Optional[bytes] = None
         if value is None:
-            return None
+            return ans
         # MySQL doesn't support tz-aware times
-        if timezone.is_aware(value):
+        if django_timezone.is_aware(value):
             if settings.USE_TZ:
                 value = value.astimezone(timezone.utc).replace(tzinfo=None)
             else:
@@ -79,18 +85,18 @@ class DatabaseOperations(MySQLDatabaseOperations):
             return datetime_to_mysql(value)
         return self.connection.converter.to_mysql(value)
 
-    def adapt_timefield_value(self, value):
+    def adapt_timefield_value(self, value: Optional[time]) -> Optional[bytes]:
         """Transform a time value to an object compatible with what is expected
         by the backend driver for time columns."""
         return self.value_to_db_time(value)
 
-    def value_to_db_time(self, value):
+    def value_to_db_time(self, value: Optional[time]) -> Optional[bytes]:
         """Convert value to MySQL TIME."""
         if value is None:
             return None
 
         # MySQL doesn't support tz-aware times
-        if timezone.is_aware(value):
+        if django_timezone.is_aware(value):
             raise ValueError("MySQL backend does not support timezone-aware times")
 
         if not self.connection.use_pure:
