@@ -1,4 +1,4 @@
-# Copyright (c) 2009, 2022, Oracle and/or its affiliates.
+# Copyright (c) 2009, 2023, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -26,18 +26,27 @@
 # along with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 
-# mypy: disable-error-code="assignment"
-
 """Implements the MySQL Client/Server protocol."""
+from __future__ import annotations
 
 import datetime
 import struct
 
+from collections import deque
 from decimal import Decimal, DecimalException
-from typing import Any, Dict, List, Optional, Sequence, Tuple, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Deque,
+    Dict,
+    List,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+)
 
 from . import utils
-from .authentication import get_auth_plugin
 from .constants import (
     PARAMETER_COUNT_AVAILABLE,
     ClientFlag,
@@ -46,20 +55,31 @@ from .constants import (
     ServerCmd,
 )
 from .errors import DatabaseError, InterfaceError, ProgrammingError, get_exception
+from .logger import logger
+from .plugins import MySQLAuthPlugin, get_auth_plugin
+from .plugins.caching_sha2_password import MySQLCachingSHA2PasswordAuthPlugin
 from .types import (
-    ConnAttrsType,
+    BinaryProtocolType,
     DescriptionType,
     EofPacketType,
     HandShakeType,
     OkPacketType,
-    ParseValueFromBinaryResultPacketTypes,
-    SocketType,
     StatsPacketType,
     StrOrBytes,
-    SupportedMysqlBinaryProtocolTypes,
 )
 
-PROTOCOL_VERSION: int = 10
+if TYPE_CHECKING:
+    from .network import MySQLSocket
+
+
+PROTOCOL_VERSION = 10
+AUTH_SWITCH_STATUS = 0xFE
+EXCHANGE_FURTHER_STATUS = 0x01
+OK_STATUS = 0x00
+MFA_STATUS = 0x02
+ERR_STATUS = 0xFF
+DEFAULT_CHARSET_ID = 45
+DEFAULT_MAX_ALLOWED_PACKET = 1073741824
 
 
 class MySQLProtocol:
@@ -69,117 +89,95 @@ class MySQLProtocol:
     """
 
     @staticmethod
-    def _connect_with_db(client_flags: int, database: Optional[str]) -> bytes:
-        """Prepare database string for handshake response"""
-        if client_flags & ClientFlag.CONNECT_WITH_DB and database:
-            return database.encode("utf8") + b"\x00"
-        return b"\x00"
+    def parse_auth_more_data(pkt: bytes) -> bytes:
+        """Parse a MySQL auth more data packet.
+
+        Args:
+            pkt: Packet representing an `auth more data` response.
+
+        Returns:
+            auth_data: Authentication method data (see [1]).
+
+        Raises:
+            InterfaceError: If packet's status tag doesn't
+                            match `protocol.EXCHANGE_FURTHER_STATUS`.
+
+        References:
+            [1]: https://dev.mysql.com/doc/dev/mysql-server/latest/\
+                page_protocol_connection_phase_packets_protocol_auth_more_data.html
+        """
+        if not pkt[4] == EXCHANGE_FURTHER_STATUS:
+            raise InterfaceError("Failed parsing AuthMoreData packet")
+        return pkt[5:]
 
     @staticmethod
-    def _auth_response(
-        client_flags: int,
-        username: Optional[StrOrBytes],
-        password: Optional[str],
-        database: Optional[str],
-        auth_plugin_class: str,
-        auth_plugin: str,
-        auth_data: Optional[bytes],
-        ssl_enabled: bool,
-    ) -> bytes:
-        """Prepare the authentication response"""
-        if not password:
-            return b"\x00"
+    def parse_auth_switch_request(pkt: bytes) -> Tuple[str, bytes]:
+        """Parse a MySQL auth switch request packet.
 
-        try:
-            auth = get_auth_plugin(auth_plugin, auth_plugin_class)(
-                auth_data,
-                username=username,
-                password=password,
-                database=database,
-                ssl_enabled=ssl_enabled,
-            )
-            plugin_auth_response = auth.auth_response()
-            # # We could receive a NULL response, an Interface error is called when so
-            # if plugin_auth_response is None:
-            #     raise InterfaceError
-        except (TypeError, InterfaceError) as err:
-            raise InterfaceError(f"Failed authentication: {err}") from err
+        Args:
+            pkt: Packet representing an `auth switch request` response.
 
-        if client_flags & ClientFlag.SECURE_CONNECTION:
-            resplen = len(plugin_auth_response)
-            auth_response = struct.pack("<B", resplen) + plugin_auth_response
-        else:
-            auth_response = plugin_auth_response + b"\x00"
-        return auth_response
+        Returns:
+            plugin_name: Name of the client authentication plugin to switch to.
+            plugin_provided_data: Plugin provided data (see [1]).
 
-    def make_auth(
-        self,
-        handshake: Optional[HandShakeType],
-        username: Optional[StrOrBytes] = None,
-        password: Optional[str] = None,
-        database: Optional[str] = None,
-        charset: int = 45,
-        client_flags: int = 0,
-        max_allowed_packet: int = 1073741824,
-        ssl_enabled: bool = False,
-        auth_plugin: Optional[str] = None,
-        conn_attrs: Optional[ConnAttrsType] = None,
-        auth_plugin_class: Optional[str] = None,
-    ) -> bytes:
-        """Make a MySQL Authentication packet"""
-        if handshake is None:
-            handshake = {}
-        try:
-            auth_data: Optional[bytes] = handshake["auth_data"]
-            auth_plugin = auth_plugin or handshake["auth_plugin"]
-        except (TypeError, KeyError) as err:
-            raise ProgrammingError(
-                f"Handshake misses authentication info ({err})"
-            ) from None
+        Raises:
+            InterfaceError: If packet's status tag doesn't
+                            match `protocol.AUTH_SWITCH_STATUS`.
 
-        if not username:
-            username = b""
-        try:
-            username_bytes = username.encode("utf8")  # type: ignore[union-attr]
-        except AttributeError:
-            # Username is already bytes
-            username_bytes = username
-
-        filler = "x" * 22
-        username_len = len(username_bytes)
-
-        packet = struct.pack(
-            f"<IIH{filler}{username_len}sx",
-            client_flags,
-            max_allowed_packet,
-            charset,
-            username_bytes,
-        )
-
-        packet += self._auth_response(
-            client_flags,
-            username,
-            password,
-            database,
-            auth_plugin_class,
-            auth_plugin,
-            auth_data,
-            ssl_enabled,
-        )
-
-        packet += self._connect_with_db(client_flags, database)
-
-        if client_flags & ClientFlag.PLUGIN_AUTH:
-            packet += auth_plugin.encode("utf8") + b"\x00"
-
-        if (client_flags & ClientFlag.CONNECT_ARGS) and conn_attrs is not None:
-            packet += self.make_conn_attrs(conn_attrs)
-
-        return packet
+        References:
+            [1]: https://dev.mysql.com/doc/dev/mysql-server/\
+                latest/page_protocol_connection_phase_packets_protocol_
+                auth_switch_request.html
+        """
+        if pkt[4] != AUTH_SWITCH_STATUS:
+            raise InterfaceError("Failed parsing AuthSwitchRequest packet")
+        pkt, plugin_name = utils.read_string(pkt[5:], end=b"\x00")
+        if pkt and pkt[-1] == 0:
+            pkt = pkt[:-1]
+        return plugin_name.decode(), pkt
 
     @staticmethod
-    def make_conn_attrs(conn_attrs: ConnAttrsType) -> bytes:
-        """Encode the connection attributes"""
+    def parse_auth_next_factor(pkt: bytes) -> Tuple[str, bytes]:
+        """Parse a MySQL auth next factor packet.
+
+        Args:
+            pkt: Packet representing an `auth next factor` response.
+
+        Returns:
+            plugin_name: Name of the client authentication plugin.
+            plugin_provided_data: Initial authentication data for that
+                                  client plugin (see [1]).
+
+        Raises:
+            InterfaceError: If packet's packet type doesn't
+                            match `protocol.MFA_STATUS`.
+
+        References:
+            [1]: https://dev.mysql.com/doc/dev/mysql-server/latest/\
+                page_protocol_connection_phase_packets_protocol_auth_\
+                next_factor_request.html
+        """
+        pkt, status = utils.read_int(pkt[4:], 1)
+        if status != MFA_STATUS:
+            raise InterfaceError("Failed parsing AuthNextFactor packet (invalid)")
+        pkt, plugin_name = utils.read_string(pkt, end=b"\x00")
+        return plugin_name.decode(), pkt
+
+    @staticmethod
+    def make_conn_attrs(conn_attrs: Dict[str, str]) -> bytes:
+        """Encode the connection attributes.
+
+        Args:
+            conn_attrs: Connection attributes.
+
+        Returns:
+            serialized_conn_attrs: Serialized connection attributes as per [1].
+
+        References:
+            [1]: https://dev.mysql.com/doc/dev/mysql-server/latest/\
+                page_protocol_connection_phase_packets_protocol_handshake_response.html
+        """
         for attr_name in conn_attrs:
             if conn_attrs[attr_name] is None:
                 conn_attrs[attr_name] = ""
@@ -189,103 +187,271 @@ class MySQLProtocol:
             + len(conn_attrs.values())
         )
 
-        conn_attrs_packet = struct.pack("<B", conn_attrs_len)
+        conn_attrs_packet = [struct.pack("<B", conn_attrs_len)]
         for attr_name in conn_attrs:
-            conn_attrs_packet += struct.pack("<B", len(attr_name))
-            conn_attrs_packet += attr_name.encode("utf8")
-            conn_attrs_packet += struct.pack("<B", len(conn_attrs[attr_name]))
-            conn_attrs_packet += conn_attrs[attr_name].encode("utf8")  # type: ignore[union-attr]
-        return conn_attrs_packet
+            conn_attrs_packet.append(struct.pack("<B", len(attr_name)))
+            conn_attrs_packet.append(attr_name.encode())
+            conn_attrs_packet.append(struct.pack("<B", len(conn_attrs[attr_name])))
+            conn_attrs_packet.append(conn_attrs[attr_name].encode())
+        return b"".join(conn_attrs_packet)
+
+    @staticmethod
+    def connect_with_db(client_flags: int, database: Optional[str]) -> bytes:
+        """Prepare database string for handshake response.
+
+        Args:
+            client_flags: Integer representing client capabilities flags.
+            database: Initial database name for the connection.
+
+        Returns:
+            serialized_database: Serialized database name as per [1].
+
+        References:
+            [1]: https://dev.mysql.com/doc/dev/mysql-server/latest/\
+                page_protocol_connection_phase_packets_protocol_handshake_response.html
+        """
+        return (
+            database.encode() + b"\x00"
+            if client_flags & ClientFlag.CONNECT_WITH_DB and database
+            else b"\x00"
+        )
+
+    @staticmethod
+    def auth_plugin_first_response(
+        auth_data: bytes,
+        username: str,
+        password: str,
+        client_flags: int,
+        auth_plugin: str,
+        auth_plugin_class: Optional[str] = None,
+        ssl_enabled: bool = False,
+        plugin_config: Optional[Dict[str, Any]] = None,
+    ) -> Tuple[bytes, MySQLAuthPlugin]:
+        """Prepare the first authentication response.
+
+        Args:
+            auth_data: Authorization data from initial handshake.
+            username: Account's username.
+            password: Account's password.
+            client_flags: Integer representing client capabilities flags.
+            auth_plugin: Authorization plugin name.
+            auth_plugin_class: Authorization plugin class (has higher precedence
+                               than the authorization plugin name).
+            ssl_enabled: Whether SSL is enabled or not.
+            plugin_config: Custom configuration to be passed to the auth plugin
+                           when invoked. The parameters defined here will override
+                           the ones defined in the auth plugin itself.
+
+        Returns:
+            auth_response: Authorization plugin response.
+            auth_strategy: Authorization plugin instance created based
+                           on the provided `auth_plugin` and `auth_plugin_class`
+                           parameters.
+
+        Raises:
+            InterfaceError: If authentication fails or when got a NULL auth response.
+        """
+        if not password:
+            # return auth response and an arbitrary auth strategy
+            return b"\x00", MySQLCachingSHA2PasswordAuthPlugin(
+                username, password, ssl_enabled=ssl_enabled
+            )
+
+        if plugin_config is None:
+            plugin_config = {}
+
+        try:
+            auth_strategy = get_auth_plugin(auth_plugin, auth_plugin_class)(
+                username, password, ssl_enabled=ssl_enabled
+            )
+            auth_response = auth_strategy.auth_response(auth_data, **plugin_config)
+        except (TypeError, InterfaceError) as err:
+            raise InterfaceError(f"Failed authentication: {err}") from err
+
+        if auth_response is None:
+            raise InterfaceError(
+                "Got NULL auth response while authenticating with "
+                f"plugin {auth_strategy.name}"
+            )
+
+        auth_response = (
+            utils.int1store(len(auth_response)) + auth_response
+            if client_flags & ClientFlag.SECURE_CONNECTION
+            else auth_response + b"\x00"
+        )
+
+        return auth_response, auth_strategy
+
+    @staticmethod
+    def make_auth(
+        handshake: HandShakeType,
+        username: str,
+        password: str,
+        database: Optional[str] = None,
+        charset: int = DEFAULT_CHARSET_ID,
+        client_flags: int = 0,
+        max_allowed_packet: int = DEFAULT_MAX_ALLOWED_PACKET,
+        auth_plugin: Optional[str] = None,
+        auth_plugin_class: Optional[str] = None,
+        conn_attrs: Optional[Dict[str, str]] = None,
+        is_change_user_request: bool = False,
+        ssl_enabled: bool = False,
+        plugin_config: Optional[Dict[str, Any]] = None,
+    ) -> Tuple[bytes, MySQLAuthPlugin]:
+        """Make a MySQL Authentication packet.
+
+        Args:
+            handshake: Initial handshake.
+            username: Account's username.
+            password: Account's password.
+            database: Initial database name for the connection
+            charset: Client charset (see [2]), only the lower 8-bits.
+            client_flags: Integer representing client capabilities flags.
+            max_allowed_packet: Maximum packet size.
+            auth_plugin: Authorization plugin name.
+            auth_plugin_class: Authorization plugin class (has higher precedence
+                               than the authorization plugin name).
+            conn_attrs: Connection attributes.
+            is_change_user_request: Whether is a `change user request` operation or not.
+            ssl_enabled: Whether SSL is enabled or not.
+            plugin_config: Custom configuration to be passed to the auth plugin
+                           when invoked. The parameters defined here will override
+                           the one defined in the auth plugin itself.
+
+        Returns:
+            handshake_response: Handshake response as per [1].
+            auth_strategy: Authorization plugin instance created based
+                           on the provided `auth_plugin` and `auth_plugin_class`.
+
+        Raises:
+            ProgrammingError: Handshake misses authentication info.
+
+        References:
+            [1]: https://dev.mysql.com/doc/dev/mysql-server/latest/\
+                page_protocol_connection_phase_packets_protocol_handshake_response.html
+
+            [2]: https://dev.mysql.com/doc/dev/mysql-server/latest/\
+                page_protocol_basic_character_set.html#a_protocol_character_set
+        """
+        b_username = username.encode()
+        response_payload = []
+
+        if is_change_user_request:
+            logger.debug("Got a `change user` request")
+
+        logger.debug("Starting authorization phase")
+        if handshake is None:
+            raise ProgrammingError("Got a NULL handshake") from None
+
+        if handshake.get("auth_data") is None:
+            raise ProgrammingError("Handshake misses authentication info") from None
+
+        try:
+            auth_plugin = auth_plugin or handshake["auth_plugin"]  # type: ignore[assignment]
+        except (TypeError, KeyError) as err:
+            raise ProgrammingError(
+                f"Handshake misses authentication plugin info ({err})"
+            ) from None
+
+        logger.debug("The provided initial strategy is %s", auth_plugin)
+
+        if is_change_user_request:
+            response_payload.append(
+                struct.pack(
+                    f"<B{len(b_username)}sx",
+                    ServerCmd.CHANGE_USER,
+                    b_username,
+                )
+            )
+        else:
+            filler = "x" * 22
+            response_payload.append(
+                struct.pack(
+                    f"<IIH{filler}{len(b_username)}sx",
+                    client_flags,
+                    max_allowed_packet,
+                    charset,
+                    b_username,
+                )
+            )
+
+        # auth plugin response
+        auth_response, auth_strategy = MySQLProtocol.auth_plugin_first_response(
+            auth_data=handshake["auth_data"],  # type: ignore[arg-type]
+            username=username,
+            password=password,
+            client_flags=client_flags,
+            auth_plugin=auth_plugin,
+            auth_plugin_class=auth_plugin_class,
+            ssl_enabled=ssl_enabled,
+            plugin_config=plugin_config,
+        )
+        response_payload.append(auth_response)
+
+        # database name
+        response_payload.append(MySQLProtocol.connect_with_db(client_flags, database))
+
+        # charset
+        if is_change_user_request:
+            response_payload.append(struct.pack("<H", charset))
+
+        # plugin name
+        if client_flags & ClientFlag.PLUGIN_AUTH:
+            response_payload.append(auth_plugin.encode() + b"\x00")
+
+        # connection attributes
+        if (client_flags & ClientFlag.CONNECT_ARGS) and conn_attrs is not None:
+            response_payload.append(MySQLProtocol.make_conn_attrs(conn_attrs))
+
+        return b"".join(response_payload), auth_strategy
 
     @staticmethod
     def make_auth_ssl(
-        charset: int = 45, client_flags: int = 0, max_allowed_packet: int = 1073741824
-    ) -> bytearray:
-        """Make a SSL authentication packet"""
-        return (
-            utils.int4store(client_flags)
-            + utils.int4store(max_allowed_packet)
-            + utils.int2store(charset)
-            + b"\x00" * 22
+        charset: int = DEFAULT_CHARSET_ID,
+        client_flags: int = 0,
+        max_allowed_packet: int = DEFAULT_MAX_ALLOWED_PACKET,
+    ) -> bytes:
+        """Make a SSL authentication packet (see [1]).
+
+        Args:
+            charset: Client charset (see [2]), only the lower 8-bits.
+            client_flags: Integer representing client capabilities flags.
+            max_allowed_packet: Maximum packet size.
+
+        Returns:
+            ssl_request_pkt: SSL connection request packet.
+
+        References:
+            [1]: https://dev.mysql.com/doc/dev/mysql-server/latest/\
+                page_protocol_connection_phase_packets_protocol_ssl_request.html
+
+            [2]: https://dev.mysql.com/doc/dev/mysql-server/latest/\
+                page_protocol_basic_character_set.html#a_protocol_character_set
+        """
+        # SSL connection request packet
+        return b"".join(
+            [
+                utils.int4store(client_flags),
+                utils.int4store(max_allowed_packet),
+                utils.int2store(charset),
+                b"\x00" * 22,
+            ]
         )
 
     @staticmethod
-    def make_command(command: int, argument: Optional[bytes] = None) -> bytearray:
+    def make_command(command: int, argument: Optional[bytes] = None) -> bytes:
         """Make a MySQL packet containing a command"""
         data = utils.int1store(command)
-        if argument is not None:
-            data += argument
-        return data
+        return data if argument is None else data + argument
 
     @staticmethod
-    def make_stmt_fetch(statement_id: int, rows: int = 1) -> bytearray:
+    def make_stmt_fetch(statement_id: int, rows: int = 1) -> bytes:
         """Make a MySQL packet with Fetch Statement command"""
         return utils.int4store(statement_id) + utils.int4store(rows)
 
-    def make_change_user(
-        self,
-        handshake: HandShakeType,
-        username: Optional[StrOrBytes] = None,
-        password: Optional[str] = None,
-        database: Optional[str] = None,
-        charset: int = 45,
-        client_flags: int = 0,
-        ssl_enabled: bool = False,
-        auth_plugin: Optional[str] = None,
-        conn_attrs: Optional[ConnAttrsType] = None,
-        auth_plugin_class: Optional[str] = None,
-    ) -> bytes:
-        """Make a MySQL packet with the Change User command"""
-
-        try:
-            auth_data: Optional[bytes] = handshake["auth_data"]
-            auth_plugin = auth_plugin or handshake["auth_plugin"]
-        except (TypeError, KeyError) as err:
-            raise ProgrammingError(
-                f"Handshake misses authentication info ({err})"
-            ) from None
-
-        if not username:
-            username = b""
-        try:
-            username_bytes = username.encode("utf8")  # type: ignore[union-attr]
-        except AttributeError:
-            # Username is already bytes
-            username_bytes = username
-
-        username_len = len(username_bytes)
-        packet = struct.pack(
-            f"<B{username_len}sx",
-            ServerCmd.CHANGE_USER,
-            username_bytes,
-        )
-
-        packet += self._auth_response(
-            client_flags,
-            username,
-            password,
-            database,
-            auth_plugin_class,
-            auth_plugin,
-            auth_data,
-            ssl_enabled,
-        )
-
-        packet += self._connect_with_db(client_flags, database)
-
-        packet += struct.pack("<H", charset)
-
-        if client_flags & ClientFlag.PLUGIN_AUTH:
-            packet += auth_plugin.encode("utf8") + b"\x00"
-
-        if (client_flags & ClientFlag.CONNECT_ARGS) and conn_attrs is not None:
-            packet += self.make_conn_attrs(conn_attrs)
-
-        return packet
-
     @staticmethod
     def parse_handshake(packet: bytes) -> HandShakeType:
-        """Parse a MySQL Handshake-packet"""
+        """Parse a MySQL Handshake-packet."""
         res = {}
         res["protocol"] = struct.unpack("<xxxxB", packet[0:5])[0]
         if res["protocol"] != PROTOCOL_VERSION:
@@ -335,15 +501,6 @@ class MySQLProtocol:
         res["capabilities"] = capabilities
         return res
 
-    @staticmethod
-    def parse_auth_next_factor(packet: bytes) -> Tuple[bytes, str]:
-        """Parse a MySQL AuthNextFactor packet."""
-        packet, status = utils.read_int(packet, 1)
-        if status != 2:
-            raise InterfaceError("Failed parsing AuthNextFactor packet (invalid)")
-        packet, auth_plugin = utils.read_string(packet, end=b"\x00")
-        return packet, auth_plugin.decode("utf-8")
-
     @staticmethod
     def parse_ok(packet: bytes) -> OkPacketType:
         """Parse a MySQL OK-packet"""
@@ -378,7 +535,7 @@ class MySQLProtocol:
 
     @staticmethod
     def parse_column(packet: bytes, encoding: str = "utf-8") -> DescriptionType:
-        """Parse a MySQL column-packet"""
+        """Parse a MySQL column-packet."""
         packet, _ = utils.read_lc_string(packet[4:])  # catalog
         packet, _ = utils.read_lc_string(packet)  # db
         packet, _ = utils.read_lc_string(packet)  # table
@@ -435,7 +592,7 @@ class MySQLProtocol:
         errmsg = "Failed getting COM_STATISTICS information"
         res: Dict[str, Union[int, Decimal]] = {}
         # Information is separated by 2 spaces
-        pairs: List[bytes] = [b""]
+        pairs = [b""]
         lbl: StrOrBytes = b""
         if with_header:
             pairs = packet[4:].split(b"\x20\x20")
@@ -459,7 +616,7 @@ class MySQLProtocol:
         return res
 
     def read_text_result(
-        self, sock: SocketType, version: Tuple[int, ...], count: int = 1
+        self, sock: MySQLSocket, version: Tuple[int, ...], count: int = 1
     ) -> Tuple[List[Tuple[Optional[bytes], ...]], Optional[EofPacketType],]:
         """Read MySQL text result
 
@@ -485,13 +642,13 @@ class MySQLProtocol:
                     datas.append(packet[4:])
                     packet = sock.recv()
                 datas.append(packet[4:])
-                rowdata = utils.read_lc_string_list(bytearray(b"").join(datas))
+                rowdata = utils.read_lc_string_list(b"".join(datas))
             elif packet[4] == 254 and packet[0] < 7:
                 eof = self.parse_eof(packet)
                 rowdata = None
             else:
                 eof = None
-                rowdata = utils.read_lc_string_list(packet[4:])
+                rowdata = utils.read_lc_string_list(bytes(packet[4:]))
             if eof is None and rowdata is not None:
                 rows.append(rowdata)
             elif eof is None and rowdata is None:
@@ -551,7 +708,7 @@ class MySQLProtocol:
     ) -> Tuple[bytes, Optional[Union[datetime.date, datetime.datetime]]]:
         """Parse a timestamp from a binary packet"""
         length = packet[0]
-        value = None
+        value: Optional[Union[datetime.datetime, datetime.date]] = None
         if length == 4:
             year = struct.unpack("<H", packet[1:3])[0]
             month = packet[3]
@@ -604,14 +761,14 @@ class MySQLProtocol:
         fields: List[DescriptionType],
         packet: bytes,
         charset: str = "utf-8",
-    ) -> Tuple[ParseValueFromBinaryResultPacketTypes, ...]:
+    ) -> Tuple[BinaryProtocolType, ...]:
         """Parse values from a binary result packet"""
         null_bitmap_length = (len(fields) + 7 + 2) // 8
         null_bitmap = [int(i) for i in packet[0:null_bitmap_length]]
         packet = packet[null_bitmap_length:]
 
         values: List[Any] = []
-        value: Any = None
+        value: BinaryProtocolType = None
         for pos, field in enumerate(fields):
             if null_bitmap[int((pos + 2) / 8)] & (1 << (pos + 2) % 8):
                 values.append(None)
@@ -641,6 +798,9 @@ class MySQLProtocol:
             elif field[1] == FieldType.TIME:
                 (packet, value) = self._parse_binary_time(packet)
                 values.append(value)
+            elif field[7] == FieldFlag.BINARY or field[8] == 63:  # "binary" charset
+                (packet, value) = utils.read_lc_string(packet)
+                values.append(value)
             else:
                 (packet, value) = utils.read_lc_string(packet)
                 try:
@@ -652,14 +812,11 @@ class MySQLProtocol:
 
     def read_binary_result(
         self,
-        sock: SocketType,
+        sock: MySQLSocket,
         columns: List[DescriptionType],
         count: int = 1,
         charset: str = "utf-8",
-    ) -> Tuple[
-        List[Tuple[ParseValueFromBinaryResultPacketTypes, ...]],
-        Optional[EofPacketType],
-    ]:
+    ) -> Tuple[List[Tuple[BinaryProtocolType, ...]], Optional[EofPacketType],]:
         """Read MySQL binary protocol result
 
         Reads all or given number of binary resultset rows from the socket.
@@ -673,7 +830,7 @@ class MySQLProtocol:
                 break
             if i == count:
                 break
-            packet = sock.recv()
+            packet = bytes(sock.recv())
             if packet[4] == 254:
                 eof = self.parse_eof(packet)
                 values = None
@@ -689,7 +846,7 @@ class MySQLProtocol:
 
     @staticmethod
     def parse_binary_prepare_ok(packet: bytes) -> Dict[str, int]:
-        """Parse a MySQL Binary Protocol OK packet"""
+        """Parse a MySQL Binary Protocol OK packet."""
         if not packet[4] == 0:
             raise InterfaceError("Failed parsing Binary OK packet")
 
@@ -742,7 +899,7 @@ class MySQLProtocol:
     @staticmethod
     def prepare_binary_timestamp(
         value: Union[datetime.date, datetime.datetime]
-    ) -> Tuple[bytearray, int]:
+    ) -> Tuple[bytes, int]:
         """Prepare a timestamp object for the MySQL binary protocol
 
         This method prepares a timestamp of type datetime.datetime or
@@ -761,29 +918,31 @@ class MySQLProtocol:
         else:
             raise ValueError("Argument must a datetime.datetime or datetime.date")
 
-        packed = (
-            utils.int2store(value.year)
-            + utils.int1store(value.month)
-            + utils.int1store(value.day)
-        )
+        chunks = [
+            utils.int2store(value.year),
+            utils.int1store(value.month),
+            utils.int1store(value.day),
+        ]
 
         if isinstance(value, datetime.datetime):
-            packed = (
-                packed
-                + utils.int1store(value.hour)
-                + utils.int1store(value.minute)
-                + utils.int1store(value.second)
+            chunks.extend(
+                [
+                    utils.int1store(value.hour),
+                    utils.int1store(value.minute),
+                    utils.int1store(value.second),
+                ]
             )
             if value.microsecond > 0:
-                packed += utils.int4store(value.microsecond)
+                chunks.append(utils.int4store(value.microsecond))
 
-        packed = utils.int1store(len(packed)) + packed
-        return (packed, field_type)
+        packed = b"".join(chunks)
+
+        return utils.int1store(len(packed)) + packed, field_type
 
     @staticmethod
     def prepare_binary_time(
         value: Union[datetime.timedelta, datetime.time]
-    ) -> Tuple[bytearray, int]:
+    ) -> Tuple[bytes, int]:
         """Prepare a time object for the MySQL binary protocol
 
         This method prepares a time object of type datetime.timedelta or
@@ -801,76 +960,82 @@ class MySQLProtocol:
         field_type = FieldType.TIME
         negative = 0
         mcs = None
-        packed = b""
+        chunks: Deque[bytes] = deque([])
 
         if isinstance(value, datetime.timedelta):
             if value.days < 0:
                 negative = 1
             (hours, remainder) = divmod(value.seconds, 3600)
             (mins, secs) = divmod(remainder, 60)
-            packed += (
-                utils.int4store(abs(value.days))
-                + utils.int1store(hours)
-                + utils.int1store(mins)
-                + utils.int1store(secs)
+            chunks.extend(
+                [
+                    utils.int4store(abs(value.days)),
+                    utils.int1store(hours),
+                    utils.int1store(mins),
+                    utils.int1store(secs),
+                ]
             )
             mcs = value.microseconds
         else:
-            packed += (
-                utils.int4store(0)
-                + utils.int1store(value.hour)
-                + utils.int1store(value.minute)
-                + utils.int1store(value.second)
+            chunks.extend(
+                [
+                    utils.int4store(0),
+                    utils.int1store(value.hour),
+                    utils.int1store(value.minute),
+                    utils.int1store(value.second),
+                ]
             )
             mcs = value.microsecond
         if mcs:
-            packed += utils.int4store(mcs)
+            chunks.append(utils.int4store(mcs))
 
-        packed = utils.int1store(negative) + packed
-        packed = utils.int1store(len(packed)) + packed
+        chunks.appendleft(utils.int1store(negative))
 
-        return (packed, field_type)
+        packed = b"".join(chunks)
+
+        return utils.int1store(len(packed)) + packed, field_type
 
     @staticmethod
-    def prepare_stmt_send_long_data(
-        statement: int, param: int, data: bytes
-    ) -> bytearray:
+    def prepare_stmt_send_long_data(statement: int, param: int, data: bytes) -> bytes:
         """Prepare long data for prepared statements
 
         Returns a string.
         """
-        packet: bytearray = utils.int4store(statement) + utils.int2store(param) + data
-        return packet
+        return b"".join([utils.int4store(statement), utils.int2store(param), data])
 
     def make_stmt_execute(
         self,
         statement_id: int,
-        data: Sequence[SupportedMysqlBinaryProtocolTypes] = (),
-        parameters: Sequence[Any] = (),
+        data: Sequence[BinaryProtocolType] = (),
+        parameters: Sequence = (),
         flags: int = 0,
         long_data_used: Optional[Dict[int, Tuple[bool]]] = None,
         charset: str = "utf8",
-        query_attrs: Optional[List[Tuple[str, Any]]] = None,
+        query_attrs: Optional[List[Tuple[str, BinaryProtocolType]]] = None,
         converter_str_fallback: bool = False,
-    ) -> bytearray:
+    ) -> bytes:
         """Make a MySQL packet with the Statement Execute command"""
         iteration_count = 1
         null_bitmap = [0] * ((len(data) + 7) // 8)
-        values = []
-        types = []
+        values: List[bytes] = []
+        types: List[bytes] = []
         packed = b""
         data_len = len(data)
-        query_attr_names = []
+        query_attr_names: List[bytes] = []
         flags = flags if not query_attrs else flags + PARAMETER_COUNT_AVAILABLE
+
         if charset == "utf8mb4":
             charset = "utf8"
+
         if long_data_used is None:
             long_data_used = {}
+
         if query_attrs:
             data = list(data)
             for _, attr_val in query_attrs:
                 data.append(attr_val)
             null_bitmap = [0] * ((len(data) + 7) // 8)
+
         if parameters or data:
             if data_len != len(parameters):
                 raise InterfaceError(
@@ -934,11 +1099,11 @@ class MySQLProtocol:
                 if query_attrs and pos + 1 > data_len:
                     name = query_attrs[pos - data_len][0].encode(charset)
                     query_attr_names.append(utils.lc_int(len(name)) + name)
-        packet = (
-            utils.int4store(statement_id)
-            + utils.int1store(flags)
-            + utils.int4store(iteration_count)
-        )
+        packet = [
+            utils.int4store(statement_id),
+            utils.int1store(flags),
+            utils.int4store(iteration_count),
+        ]
 
         # if (num_params > 0 || (CLIENT_QUERY_ATTRIBUTES \
         #                        && (flags & PARAMETER_COUNT_AVAILABLE)) {
@@ -949,46 +1114,26 @@ class MySQLProtocol:
         if parameter_count:
             # if CLIENT_QUERY_ATTRIBUTES is on
             if query_attrs is not None:
-                packet += utils.lc_int(parameter_count)
+                packet.append(utils.lc_int(parameter_count))
+
+            packet.extend([struct.pack("B", bit) for bit in null_bitmap])
+            packet.append(utils.int1store(1))
 
-            packet += b"".join(
-                [struct.pack("B", bit) for bit in null_bitmap]
-            ) + utils.int1store(1)
             count = 0
             for a_type in types:
-                packet += a_type
+                packet.append(a_type)
                 # if CLIENT_QUERY_ATTRIBUTES is on {
                 #    string<lenenc>    parameter_name    Name of the parameter
                 # or empty if not present
                 # } if CLIENT_QUERY_ATTRIBUTES is on
                 if query_attrs is not None:
                     if count + 1 > data_len:
-                        packet += query_attr_names[count - data_len]
+                        packet.append(query_attr_names[count - data_len])
                     else:
-                        packet += b"\x00"
+                        packet.append(b"\x00")
                 count += 1
 
             for a_value in values:
-                packet += a_value
+                packet.append(a_value)
 
-        return packet
-
-    @staticmethod
-    def parse_auth_switch_request(packet: bytes) -> Tuple[str, bytes]:
-        """Parse a MySQL AuthSwitchRequest-packet"""
-        if not packet[4] == 254:
-            raise InterfaceError("Failed parsing AuthSwitchRequest packet")
-
-        packet, plugin_name = utils.read_string(packet[5:], end=b"\x00")
-        if packet and packet[-1] == 0:
-            packet = packet[:-1]
-
-        return plugin_name.decode("utf8"), packet
-
-    @staticmethod
-    def parse_auth_more_data(packet: bytes) -> bytes:
-        """Parse a MySQL AuthMoreData-packet"""
-        if not packet[4] == 1:
-            raise InterfaceError("Failed parsing AuthMoreData packet")
-
-        return packet[5:]
+        return b"".join(packet)
