@@ -1,4 +1,4 @@
-# Copyright (c) 2016, 2022, Oracle and/or its affiliates.
+# Copyright (c) 2016, 2023, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -28,19 +28,23 @@
 
 """Implementation of the Result classes."""
 
+from __future__ import annotations
+
 import decimal
 import struct
 import sys
 
 from datetime import datetime, timedelta
+from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
 
 from .charsets import MYSQL_CHARACTER_SETS
 from .dbdoc import DbDoc
 from .helpers import decode_from_bytes, deprecated
+from .types import ConnectionType, FieldTypes
 
 
 # pylint: disable=missing-class-docstring,missing-function-docstring
-def from_protobuf(column, payload):
+def from_protobuf(column: Column, payload: bytes) -> Any:
     if len(payload) == 0:
         return None
 
@@ -51,26 +55,26 @@ def from_protobuf(column, payload):
         return ColumnProtoType.converter_map[column.get_proto_type()](payload)
     except KeyError as err:
         sys.stderr.write(f"{err}")
-        sys.stderr.write(f"{payload.encode('hex')}")
+        sys.stderr.write(f"{payload.encode('hex')}")  # type: ignore[attr-defined]
         return None
 
 
-def bytes_from_protobuf(payload):
+def bytes_from_protobuf(payload: bytes) -> bytes:
     # Strip trailing char
     return payload[:-1]
 
 
-def float_from_protobuf(payload):
+def float_from_protobuf(payload: bytes) -> float:
     assert len(payload) == 4
     return struct.unpack("<f", payload)[0]
 
 
-def double_from_protobuf(payload):
+def double_from_protobuf(payload: bytes) -> float:
     assert len(payload) == 8
     return struct.unpack("<d", payload)[0]
 
 
-def varint_from_protobuf_stream(payload):
+def varint_from_protobuf_stream(payload: bytes) -> Tuple[int, bytes]:
     if len(payload) == 0:
         raise ValueError("Payload is empty")
 
@@ -79,7 +83,7 @@ def varint_from_protobuf_stream(payload):
     shift = 0
 
     for item in payload:
-        char = item if isinstance(item, int) else ord(item)
+        char = item if isinstance(item, int) else ord(item)  # type: ignore[arg-type]
         eos = (char & 0x80) == 0
         cur_bits = char & 0x7F
         cur_bits <<= shift
@@ -92,7 +96,7 @@ def varint_from_protobuf_stream(payload):
     raise EOFError("Payload too short")
 
 
-def varint_from_protobuf(payload):
+def varint_from_protobuf(payload: bytes) -> int:
     i, payload = varint_from_protobuf_stream(payload)
     if len(payload) != 0:
         raise ValueError("Payload too long")
@@ -100,7 +104,7 @@ def varint_from_protobuf(payload):
     return i
 
 
-def varsint_from_protobuf(payload):
+def varsint_from_protobuf(payload: bytes) -> int:
     i, payload = varint_from_protobuf_stream(payload)
     if len(payload) != 0:
         raise ValueError("Payload too long")
@@ -116,8 +120,8 @@ def varsint_from_protobuf(payload):
     return i
 
 
-def set_from_protobuf(payload):
-    set_pb = []
+def set_from_protobuf(payload: bytes) -> List[bytes]:
+    set_pb: List = []
     while True:
         try:
             field_len, payload = varint_from_protobuf_stream(payload)
@@ -137,14 +141,14 @@ def set_from_protobuf(payload):
     return set_pb
 
 
-def decimal_from_protobuf(payload):
+def decimal_from_protobuf(payload: bytes) -> decimal.Decimal:
     digits = []
     sign = None
-    scale = payload[0] if isinstance(payload[0], int) else ord(payload[0])
+    scale = payload[0] if isinstance(payload[0], int) else ord(payload[0])  # type: ignore[arg-type]
     payload = payload[1:]
 
     for item in payload:
-        char = item if isinstance(item, int) else ord(item)
+        char = item if isinstance(item, int) else ord(item)  # type: ignore[arg-type]
         high_bcd = (char & 0xF0) >> 4
         low_bcd = char & 0x0F
         if high_bcd < 0x0A:
@@ -173,7 +177,7 @@ def decimal_from_protobuf(payload):
     return decimal.Decimal((sign, digits, -scale))
 
 
-def datetime_from_protobuf(payload):
+def datetime_from_protobuf(payload: bytes) -> datetime:
     # A sequence of varints
     hour = 0
     minutes = 0
@@ -194,7 +198,7 @@ def datetime_from_protobuf(payload):
     return datetime(year, month, day, hour, minutes, seconds, useconds)
 
 
-def time_from_protobuf(payload):
+def time_from_protobuf(payload: bytes) -> timedelta:
     # A sequence of varints
     hour = 0
     minutes = 0
@@ -269,32 +273,32 @@ class ColumnType:
     LONGTEXT = 35
 
     @classmethod
-    def to_string(cls, needle):
+    def to_string(cls, needle: Any) -> Optional[str]:
         for key, value in vars(cls).items():
             if value == needle:
                 return key
         return None
 
     @classmethod
-    def from_string(cls, key):
+    def from_string(cls, key: str) -> Any:
         return getattr(cls, key.upper(), None)
 
     @classmethod
-    def is_char(cls, col_type):
+    def is_char(cls, col_type: int) -> bool:
         return col_type in (
             cls.CHAR,
             cls.VARCHAR,
         )
 
     @classmethod
-    def is_binary(cls, col_type):
+    def is_binary(cls, col_type: int) -> bool:
         return col_type in (
             cls.BINARY,
             cls.VARBINARY,
         )
 
     @classmethod
-    def is_text(cls, col_type):
+    def is_text(cls, col_type: int) -> bool:
         return col_type in (
             cls.TEXT,
             cls.TINYTEXT,
@@ -303,7 +307,7 @@ class ColumnType:
         )
 
     @classmethod
-    def is_decimals(cls, col_type):
+    def is_decimals(cls, col_type: int) -> bool:
         return col_type in (
             cls.REAL,
             cls.DOUBLE,
@@ -313,7 +317,7 @@ class ColumnType:
         )
 
     @classmethod
-    def is_numeric(cls, col_type):
+    def is_numeric(cls, col_type: int) -> bool:
         return col_type in (
             cls.BIT,
             cls.TINYINT,
@@ -324,7 +328,7 @@ class ColumnType:
         )
 
     @classmethod
-    def is_finite_set(cls, col_type):
+    def is_finite_set(cls, col_type: int) -> bool:
         return col_type in (
             cls.SET,
             cls.ENUM,
@@ -344,7 +348,7 @@ class ColumnProtoType:
     BIT = 17
     DECIMAL = 18
 
-    converter_map = {
+    converter_map: Dict[int, Callable[[bytes], Any]] = {
         SINT: varsint_from_protobuf,
         UINT: varint_from_protobuf,
         BYTES: bytes_from_protobuf,
@@ -360,18 +364,18 @@ class ColumnProtoType:
 
 
 class Flags:
-    def __init__(self, value):
-        self._allowed_flags = {}
-        self._flag_names = {}
+    def __init__(self, value: int) -> None:
+        self._allowed_flags: Dict[str, int] = {}
+        self._flag_names: Dict[int, str] = {}
         for key, val in self.__class__.__dict__.items():
             if key.startswith("__"):
                 continue
             if isinstance(val, int):
                 self._allowed_flags[key] = val
                 self._flag_names[val] = key
-        self._value = value
+        self._value: int = value
 
-    def __str__(self):
+    def __str__(self) -> str:
         mask = 1
         flag_names = []
         value = self._value
@@ -390,11 +394,11 @@ class Flags:
         return ",".join(flag_names)
 
     @property
-    def value(self):
+    def value(self) -> int:
         return self._value
 
     @value.setter
-    def value(self, val):
+    def value(self, val: int) -> None:
         self._value = val
 
 
@@ -457,39 +461,39 @@ class Column:
 
     def __init__(
         self,
-        col_type,
-        catalog=None,
-        schema=None,
-        table=None,
-        original_table=None,
-        name=None,
-        original_name=None,
-        length=None,
-        collation=None,
-        fractional_digits=None,
-        flags=None,
-        content_type=None,
-    ):
-        self._schema = decode_from_bytes(schema)
-        self._name = decode_from_bytes(name)
-        self._original_name = decode_from_bytes(original_name)
-        self._table = decode_from_bytes(table)
-        self._original_table = decode_from_bytes(original_table)
-        self._proto_type = col_type
-        self._col_type = None
-        self._catalog = catalog
-        self._length = length
-        self._collation = collation
-        self._fractional_digits = fractional_digits
-        self._flags = flags
-        self._content_type = content_type
-        self._number_signed = False
-        self._is_padded = False
-        self._is_binary = False
-        self._is_bytes = False
-        self._collation_name = None
-        self._character_set_name = None
-        self._zero_fill = None
+        col_type: int,
+        catalog: Optional[str] = None,
+        schema: Optional[str] = None,
+        table: Optional[str] = None,
+        original_table: Optional[str] = None,
+        name: Optional[str] = None,
+        original_name: Optional[str] = None,
+        length: Optional[int] = None,
+        collation: Optional[int] = None,
+        fractional_digits: Optional[int] = None,
+        flags: Optional[int] = None,
+        content_type: Optional[int] = None,
+    ) -> None:
+        self._schema: str = decode_from_bytes(schema)
+        self._name: str = decode_from_bytes(name)
+        self._original_name: str = decode_from_bytes(original_name)
+        self._table: str = decode_from_bytes(table)
+        self._original_table: str = decode_from_bytes(original_table)
+        self._proto_type: int = col_type
+        self._col_type: Optional[int] = None
+        self._catalog: Optional[str] = catalog
+        self._length: Optional[int] = length
+        self._collation: Optional[int] = collation
+        self._fractional_digits: Optional[int] = fractional_digits
+        self._flags: Optional[int] = flags
+        self._content_type: Optional[int] = content_type
+        self._number_signed: bool = False
+        self._is_padded: Union[bool, int] = False
+        self._is_binary: bool = False
+        self._is_bytes: bool = False
+        self._collation_name: Optional[str] = None
+        self._character_set_name: Optional[str] = None
+        self._zero_fill: Optional[int] = None
 
         if self._collation > 0:
             if self._collation >= len(MYSQL_CHARACTER_SETS):
@@ -509,7 +513,7 @@ class Column:
             ColumnType.STRING,
         )
 
-    def __str__(self):
+    def __str__(self) -> str:
         return str(
             {
                 "col_type": self._col_type,
@@ -519,7 +523,7 @@ class Column:
             }
         )
 
-    def _map_bytes(self):
+    def _map_bytes(self) -> None:
         """Map bytes."""
         if self._content_type == BytesContentType.GEOMETRY:
             self._col_type = ColumnType.GEOMETRY
@@ -533,7 +537,7 @@ class Column:
             self._col_type = ColumnType.STRING
         self._is_padded = self._flags & 1
 
-    def _map_datetime(self):
+    def _map_datetime(self) -> None:
         """Map datetime."""
         if self._length == 10:
             self._col_type = ColumnType.DATE
@@ -544,7 +548,7 @@ class Column:
         else:
             raise ValueError("Datetime mapping scenario unhandled")
 
-    def _map_int_type(self):
+    def _map_int_type(self) -> None:
         """Map int type."""
         if self._length <= 4:
             self._col_type = ColumnType.TINYINT
@@ -558,7 +562,7 @@ class Column:
             self._col_type = ColumnType.BIGINT
         self._number_signed = True
 
-    def _map_uint_type(self):
+    def _map_uint_type(self) -> None:
         """Map uint type."""
         if self._length <= 3:
             self._col_type = ColumnType.TINYINT
@@ -572,7 +576,7 @@ class Column:
             self._col_type = ColumnType.BIGINT
         self._zero_fill = self._flags & 1
 
-    def _map_type(self):
+    def _map_type(self) -> None:
         """Map type."""
         if self._proto_type == ColumnProtoType.SINT:
             self._map_int_type()
@@ -603,7 +607,7 @@ class Column:
             raise ValueError(f"Unknown column type {self._proto_type}")
 
     @property
-    def schema_name(self):
+    def schema_name(self) -> str:
         """str: The schema name.
 
         .. versionadded:: 8.0.12
@@ -611,7 +615,7 @@ class Column:
         return self._schema
 
     @property
-    def table_name(self):
+    def table_name(self) -> str:
         """str: The table name.
 
         .. versionadded:: 8.0.12
@@ -619,7 +623,7 @@ class Column:
         return self._original_table or self._table
 
     @property
-    def table_label(self):
+    def table_label(self) -> str:
         """str: The table label.
 
         .. versionadded:: 8.0.12
@@ -627,7 +631,7 @@ class Column:
         return self._table or self._original_table
 
     @property
-    def column_name(self):
+    def column_name(self) -> str:
         """str: The column name.
 
         .. versionadded:: 8.0.12
@@ -635,7 +639,7 @@ class Column:
         return self._original_name or self._name
 
     @property
-    def column_label(self):
+    def column_label(self) -> str:
         """str: The column label.
 
         .. versionadded:: 8.0.12
@@ -643,7 +647,7 @@ class Column:
         return self._name or self._original_name
 
     @property
-    def type(self):
+    def type(self) -> int:
         """int: The column type.
 
         .. versionadded:: 8.0.12
@@ -651,7 +655,7 @@ class Column:
         return self._col_type
 
     @property
-    def length(self):
+    def length(self) -> int:
         """int. The column length.
 
         .. versionadded:: 8.0.12
@@ -659,7 +663,7 @@ class Column:
         return self._length
 
     @property
-    def fractional_digits(self):
+    def fractional_digits(self) -> int:
         """int: The column fractional digits.
 
         .. versionadded:: 8.0.12
@@ -667,7 +671,7 @@ class Column:
         return self._fractional_digits
 
     @property
-    def collation_name(self):
+    def collation_name(self) -> str:
         """str: The collation name.
 
         .. versionadded:: 8.0.12
@@ -675,14 +679,14 @@ class Column:
         return self._collation_name
 
     @property
-    def character_set_name(self):
+    def character_set_name(self) -> str:
         """str: The character set name.
 
         .. versionadded:: 8.0.12
         """
         return self._character_set_name
 
-    def get_schema_name(self):
+    def get_schema_name(self) -> str:
         """Returns the schema name.
 
         Returns:
@@ -690,7 +694,7 @@ class Column:
         """
         return self._schema
 
-    def get_table_name(self):
+    def get_table_name(self) -> str:
         """Returns the table name.
 
         Returns:
@@ -698,7 +702,7 @@ class Column:
         """
         return self._original_table or self._table
 
-    def get_table_label(self):
+    def get_table_label(self) -> str:
         """Returns the table label.
 
         Returns:
@@ -706,7 +710,7 @@ class Column:
         """
         return self._table or self._original_table
 
-    def get_column_name(self):
+    def get_column_name(self) -> str:
         """Returns the column name.
 
         Returns:
@@ -714,7 +718,7 @@ class Column:
         """
         return self._original_name or self._name
 
-    def get_column_label(self):
+    def get_column_label(self) -> str:
         """Returns the column label.
 
         Returns:
@@ -722,7 +726,7 @@ class Column:
         """
         return self._name or self._original_name
 
-    def get_proto_type(self):
+    def get_proto_type(self) -> int:
         """Returns the column proto type.
 
         Returns:
@@ -730,7 +734,7 @@ class Column:
         """
         return self._proto_type
 
-    def get_type(self):
+    def get_type(self) -> int:
         """Returns the column type.
 
         Returns:
@@ -738,7 +742,7 @@ class Column:
         """
         return self._col_type
 
-    def get_length(self):
+    def get_length(self) -> int:
         """Returns the column length.
 
         Returns:
@@ -746,7 +750,7 @@ class Column:
         """
         return self._length
 
-    def get_fractional_digits(self):
+    def get_fractional_digits(self) -> int:
         """Returns the column fractional digits.
 
         Returns:
@@ -754,7 +758,7 @@ class Column:
         """
         return self._fractional_digits
 
-    def get_collation_name(self):
+    def get_collation_name(self) -> str:
         """Returns the collation name.
 
         Returns:
@@ -762,7 +766,7 @@ class Column:
         """
         return self._collation_name
 
-    def get_character_set_name(self):
+    def get_character_set_name(self) -> str:
         """Returns the character set name.
 
         Returns:
@@ -770,7 +774,7 @@ class Column:
         """
         return self._character_set_name
 
-    def is_number_signed(self):
+    def is_number_signed(self) -> bool:
         """Returns `True` if is a number signed.
 
         Returns:
@@ -778,7 +782,7 @@ class Column:
         """
         return self._number_signed
 
-    def is_padded(self):
+    def is_padded(self) -> Union[bool, int]:
         """Returns `True` if is padded.
 
         Returns:
@@ -786,7 +790,7 @@ class Column:
         """
         return self._is_padded
 
-    def is_bytes(self):
+    def is_bytes(self) -> bool:
         """Returns `True` if is bytes.
 
         Returns:
@@ -803,14 +807,16 @@ class Row:
         fields (`list`): The list of fields.
     """
 
-    def __init__(self, resultset, fields):
-        self._fields = fields
-        self._resultset = resultset
+    def __init__(
+        self, resultset: Union[BufferingResult, RowResult], fields: Sequence[FieldTypes]
+    ) -> None:
+        self._fields: Sequence[FieldTypes] = fields
+        self._resultset: Union[BufferingResult, RowResult] = resultset
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return repr(self._fields)
 
-    def __getitem__(self, index):
+    def __getitem__(self, index: Union[int, str]) -> Any:
         """Returns the value of a column by name or index.
 
         .. versionchanged:: 8.0.12
@@ -823,7 +829,7 @@ class Row:
         return self._fields[int_index]
 
     @deprecated("8.0.12")
-    def get_string(self, str_index):
+    def get_string(self, str_index: str) -> str:
         """Returns the value using the column name.
 
         Args:
@@ -846,13 +852,13 @@ class BaseResult:
         connection (mysqlx.connection.Connection): The Connection object.
     """
 
-    def __init__(self, connection):
-        self._connection = connection
-        self._closed = False
-        self._rows_affected = 0
-        self._generated_id = -1
-        self._generated_ids = []
-        self._warnings = []
+    def __init__(self, connection: ConnectionType) -> None:
+        self._connection: ConnectionType = connection
+        self._closed: bool = False
+        self._rows_affected: int = 0
+        self._generated_id: int = -1
+        self._generated_ids: List[int] = []
+        self._warnings: List[Dict[str, Union[int, str]]] = []
 
         if connection is None:
             self._protocol = None
@@ -860,7 +866,7 @@ class BaseResult:
             self._protocol = connection.protocol
             connection.fetch_active_result()
 
-    def get_affected_items_count(self):
+    def get_affected_items_count(self) -> int:
         """Returns the number of affected items for the last operation.
 
         Returns:
@@ -868,7 +874,7 @@ class BaseResult:
         """
         return self._rows_affected
 
-    def get_warnings(self):
+    def get_warnings(self) -> List[Dict[str, Union[int, str]]]:
         """Returns the warnings.
 
         Returns:
@@ -876,7 +882,7 @@ class BaseResult:
         """
         return self._warnings
 
-    def get_warnings_count(self):
+    def get_warnings_count(self) -> int:
         """Returns the number of warnings.
 
         Returns:
@@ -884,11 +890,11 @@ class BaseResult:
         """
         return len(self._warnings)
 
-    def set_closed(self, flag):
+    def set_closed(self, flag: bool) -> None:
         """Sets if resultset fetch is done."""
         self._closed = flag
 
-    def append_warning(self, level, code, msg):
+    def append_warning(self, level: int, code: int, msg: str) -> None:
         """Append a warning.
 
         Args:
@@ -898,15 +904,15 @@ class BaseResult:
         """
         self._warnings.append({"level": level, "code": code, "msg": msg})
 
-    def set_generated_ids(self, generated_ids):
+    def set_generated_ids(self, generated_ids: List[int]) -> None:
         """Sets the generated ids."""
         self._generated_ids = generated_ids
 
-    def set_generated_insert_id(self, generated_id):
+    def set_generated_insert_id(self, generated_id: int) -> None:
         """Sets the generated insert id."""
         self._generated_id = generated_id
 
-    def set_rows_affected(self, total):
+    def set_rows_affected(self, total: int) -> None:
         """Sets the number of rows affected."""
         self._rows_affected = total
 
@@ -920,14 +926,18 @@ class Result(BaseResult):
                                                    ids (`list`): A list of IDs.
     """
 
-    def __init__(self, connection=None, ids=None):
+    def __init__(
+        self,
+        connection: Optional[ConnectionType] = None,
+        ids: Optional[List[int]] = None,
+    ) -> None:
         super().__init__(connection)
-        self._ids = ids
+        self._ids: Optional[List[int]] = ids
 
         if connection is not None:
             self._connection.close_result(self)
 
-    def get_autoincrement_value(self):
+    def get_autoincrement_value(self) -> int:
         """Returns the last insert id auto generated.
 
         Returns:
@@ -936,7 +946,7 @@ class Result(BaseResult):
         return self._generated_id
 
     @deprecated("8.0.12")
-    def get_document_id(self):
+    def get_document_id(self) -> Optional[int]:
         """Returns ID of the last document inserted into a collection.
 
         .. deprecated:: 8.0.12
@@ -946,14 +956,14 @@ class Result(BaseResult):
         return self._ids[0]
 
     @deprecated("8.0.12")
-    def get_generated_insert_id(self):
+    def get_generated_insert_id(self) -> int:
         """Returns the generated insert id.
 
         .. deprecated:: 8.0.12
         """
         return self._generated_id
 
-    def get_generated_ids(self):
+    def get_generated_ids(self) -> List[int]:
         """Returns the generated ids."""
         return self._generated_ids
 
@@ -966,25 +976,25 @@ class BufferingResult(BaseResult):
                                                    ids (`list`): A list of IDs.
     """
 
-    def __init__(self, connection):
+    def __init__(self, connection: ConnectionType) -> None:
         super().__init__(connection)
-        self._columns = []
-        self._has_data = False
-        self._has_more_results = False
-        self._items = []
-        self._page_size = 0
-        self._position = -1
+        self._columns: List[Column] = []
+        self._has_data: bool = False
+        self._has_more_results: bool = False
+        self._items: List[Union[Row, DbDoc]] = []
+        self._page_size: int = 0
+        self._position: int = -1
         self._init_result()
 
-    def __getitem__(self, index):
+    def __getitem__(self, index: int) -> Union[Row, DbDoc]:
         return self._items[index]
 
     @property
-    def count(self):
+    def count(self) -> int:
         """int: The total of items."""
         return len(self._items)
 
-    def _init_result(self):
+    def _init_result(self) -> None:
         """Initialize the result."""
         self._columns = self._connection.get_column_metadata(self)
         self._has_more_data = len(self._columns) > 0
@@ -993,7 +1003,7 @@ class BufferingResult(BaseResult):
         self._position = -1
         self._connection.set_active_result(self if self._has_more_data else None)
 
-    def _read_item(self, dumping):
+    def _read_item(self, dumping: bool) -> Optional[Union[Row, DbDoc]]:
         """Read item.
 
         Args:
@@ -1012,7 +1022,7 @@ class BufferingResult(BaseResult):
                 item[key] = from_protobuf(column, row["field"][key])
         return Row(self, item)
 
-    def _page_in_items(self):
+    def _page_in_items(self) -> Union[bool, int]:
         """Reads the page items.
 
         Returns:
@@ -1030,7 +1040,7 @@ class BufferingResult(BaseResult):
             count += 1
         return count
 
-    def index_of(self, col_name):
+    def index_of(self, col_name: str) -> int:
         """Returns the index of the column.
 
         Returns:
@@ -1043,7 +1053,7 @@ class BufferingResult(BaseResult):
             index += 1
         return -1
 
-    def fetch_one(self):
+    def fetch_one(self) -> Optional[Union[Row, DbDoc]]:
         """Fetch one item.
 
         Returns:
@@ -1054,7 +1064,7 @@ class BufferingResult(BaseResult):
 
         return self._read_item(False)
 
-    def fetch_all(self):
+    def fetch_all(self) -> List[Union[Row, DbDoc]]:
         """Fetch all items.
 
         Returns:
@@ -1066,7 +1076,7 @@ class BufferingResult(BaseResult):
                 break
         return self._items
 
-    def set_has_data(self, flag):
+    def set_has_data(self, flag: bool) -> None:
         """Sets if result has data.
 
         Args:
@@ -1074,7 +1084,7 @@ class BufferingResult(BaseResult):
         """
         self._has_data = flag
 
-    def set_has_more_results(self, flag):
+    def set_has_more_results(self, flag: bool) -> None:
         """Sets if has more results.
 
         Args:
@@ -1091,11 +1101,11 @@ class RowResult(BufferingResult):
     """
 
     @property
-    def columns(self):
+    def columns(self) -> List[Column]:
         """`list`: The list of columns."""
         return self._columns
 
-    def get_columns(self):
+    def get_columns(self) -> List[Column]:
         """Returns the list of columns.
 
         Returns:
@@ -1113,7 +1123,7 @@ class SqlResult(RowResult):
         connection (mysqlx.connection.Connection): The Connection object.
     """
 
-    def get_autoincrement_value(self):
+    def get_autoincrement_value(self) -> int:
         """Returns the identifier for the last record inserted.
 
         Returns:
@@ -1121,7 +1131,7 @@ class SqlResult(RowResult):
         """
         return self._generated_id
 
-    def next_result(self):
+    def next_result(self) -> bool:
         """Process the next result.
 
         Returns:
@@ -1133,7 +1143,7 @@ class SqlResult(RowResult):
         self._init_result()
         return True
 
-    def has_data(self):
+    def has_data(self) -> bool:
         """Returns True if result has data.
 
         Returns:
@@ -1152,7 +1162,7 @@ class DocResult(BufferingResult):
         connection (mysqlx.connection.Connection): The Connection object.
     """
 
-    def _read_item(self, dumping):
+    def _read_item(self, dumping: bool) -> DbDoc:
         """Read item.
 
         Args:
@@ -1164,4 +1174,4 @@ class DocResult(BufferingResult):
         row = super()._read_item(dumping)
         if row is None:
             return None
-        return DbDoc(decode_from_bytes(row[0]))
+        return DbDoc(decode_from_bytes(row[0]))  # type: ignore[index]
